<?php

namespace Drupal\searchstax\Controller;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Route;

/**
 * Provides a controller for SearchStax pages.
 */
class SearchPageController extends ControllerBase {

  /**
   * Returns the routes provided by this controller.
   *
   * @return \Symfony\Component\Routing\Route[]
   *   The routes provided by this controller.
   */
  public function routes(): array {
    $config = $this->config('searchstax.settings');
    $path = $config->get('search_page_path');
    if (!$config->get('search_page_enable') || !$path) {
      return [];
    }
    return [
      'searchstax.search_page' => new Route(
        $path,
        [
          '_controller' => '\Drupal\searchstax\Controller\SearchPageController::displaySearchPage',
        ],
        [
          '_access' => 'TRUE',
        ]
      ),
    ];
  }

  /**
   * Page callback: Displays the SearchStudio JS search page.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request for this page.
   *
   * @return array
   *   A render array.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   Thrown if the search page isn't configured to be shown.
   */
  public function displaySearchPage(Request $request): array {
    $config = $this->config('searchstax.settings');
    $js_config = trim($config->get('search_page_js_config') ?: '');
    if (!$config->get('search_page_enable') || !$js_config) {
      throw new NotFoundHttpException();
    }

    $build = [];
    CacheableMetadata::createFromObject($config)->applyTo($build);

    $build['search_box'] = [
      '#type' => 'markup',
      '#markup' => '<div id="searchStudioContainer"></div>',
    ];
    $build['feedback_link'] = [
      '#type' => 'markup',
      '#markup' => '<div id="sf-feedback"></div>',
    ];

    $session_placeholder = '"[user:session]"';
    if (strpos($js_config, $session_placeholder) !== FALSE) {
      if ($request->hasPreviousSession()) {
        $session = Json::encode($request->getSession()->getId());
      }
      else {
        $function_def = <<<'END'
function get_random_session_id() {
  var chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
  var session = '';
  for (var i = 0; i != 32; i++) {
    session += chars.charAt(Math.floor(Math.random() * chars.length));
  }
  return session;
}


END;
        $js_config = $function_def . $js_config;
        $session = 'get_random_session_id()';
      }
      $js_config = str_replace($session_placeholder, $session, $js_config);
    }
    $build['#attached']['html_head'][] = [
      [
        '#tag' => 'script',
        '#attributes' => [
          'type' => 'text/javascript',
        ],
        '#value' => $js_config,
      ],
      'searchstax_search_config_script',
    ];
    $build['#attached']['library'][] = 'searchstax/searchstax.search_page';

    return $build;
  }

}
