<?php

namespace Drupal\searchstax\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\search_api\Display\DisplayPluginManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for changing the module's settings.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The plugin manager search api display.
   *
   * @var \Drupal\search_api\Display\DisplayPluginManagerInterface|null
   */
  protected $displayPluginManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $form = parent::create($container);

    $form->setDisplayPluginManager($container->get('plugin.manager.search_api.display'));

    return $form;
  }

  /**
   * Retrieves the plugin manager search api display.
   *
   * @return \Drupal\search_api\Display\DisplayPluginManagerInterface
   *   The plugin manager search api display.
   */
  public function getDisplayPluginManager(): DisplayPluginManagerInterface {
    return $this->displayPluginManager ?: \Drupal::service('plugin.manager.search_api.display');
  }

  /**
   * Sets the plugin manager search api display.
   *
   * @param \Drupal\search_api\Display\DisplayPluginManagerInterface $display_plugin_manager
   *   The new plugin manager search api display.
   *
   * @return $this
   */
  public function setDisplayPluginManager(DisplayPluginManagerInterface $display_plugin_manager): self {
    $this->displayPluginManager = $display_plugin_manager;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'searchstax_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [
      'searchstax.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('searchstax.settings');

    $form['analytics_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Global analytics key'),
      '#description' => $this->t('The SearchStudio analytics key associated with this website. You can override this on a per-search basis below, if you have more than one search app configured for this site.'),
      '#default_value' => $config->get('analytics_key'),
    ];

    $form['search_specific_analytics_keys'] = [
      '#type' => 'details',
      '#title' => $this->t('Search-specific analytics keys'),
      '#description' => $this->t('If you have multiple search apps configured in SearchStax for this website, please enter the analytics key to use for each individual search below. Leave a field blank to use the global key set above for that search – or disable tracking for a search, in case you have left the global key empty, too.'),
      '#tree' => TRUE,
    ];
    $displays = $this->getDisplayPluginManager()->getInstances();
    foreach ($displays as $id => $display) {
      $form['search_specific_analytics_keys'][$id] = [
        '#type' => 'textfield',
        '#title' => $display->label(),
        '#default_value' => $config->get("search_specific_analytics_keys.$id"),
      ];
    }

    if (\Drupal::moduleHandler()->moduleExists('search_api_autocomplete')) {
      $form['autosuggest_core'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Auto-suggest core'),
        '#description' => $this->t('The “core” part of the your SearchStudio auto-suggest URL. This is the second-to-last part of the URL, between “solr” and “emsuggest”: [DOMAIN]/solr/[CORE]/emsuggest. (Only needed if you want to use auto-suggest.'),
        '#default_value' => $config->get('autosuggest_core'),
      ];
    }
    // @todo Check whether /emselect is available before allowing to enable?
    $form['searches_via_searchstudio'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Re-route searches through SearchStudio'),
      '#description' => $this->t('Re-route all Solr searches through the SearchStudio search handler, to immediately apply adjustments you make in SearchStudio.'),
      '#default_value' => (bool) $config->get('searches_via_searchstudio'),
    ];
    $form['search_page_enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable the SearchStudio search page'),
      '#description' => $this->t("Check to enable SearchStudio's built-in search page, powered by Javascript and Solr directly."),
      '#default_value' => (bool) $config->get('search_page_enable'),
    ];
    $states = [
      'visible' => [
        ':input[name="search_page_enable"]' => ['checked' => TRUE],
      ],
    ];
    $form['search_page'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Search page configuration'),
      '#tree' => FALSE,
      '#states' => $states,
    ];
    $form['search_page']['search_page_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Path'),
      '#description' => $this->t('The internal path where the search page should be displayed. Please include a leading slash. (For instance: "/search-page".'),
      '#default_value' => $config->get('search_page_path'),
    ];
    $form['search_page']['search_page_js_config'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Javascript config'),
      '#description' => $this->t('The Javascript configuration for the search page. Please see the "StudioJS Integration Guide" for details. This will be included verbatim on the search page as Javascript and has to at least define the "studioConfig" constant. You can use <code>"[user:session]"</code> for the session value, it will be replaced by an appropriate value.<br /><strong>Warning:</strong> Do not paste text from untrusted sources into this field, as this could damage your web site or its visitors.'),
      '#default_value' => $config->get('search_page_js_config'),
      '#rows' => 10,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    if (!empty($form['autosuggest_core'])
        && strpos($form_state->getValue('autosuggest_core'), '/') !== FALSE) {
      $vars = ['@name' => $form['autosuggest_core']['#title']];
      $message = $this->t('Please enter @name without any slashes (/).', $vars);
      $form_state->setError($form['autosuggest_core'], $message);
    }

    if (!$form_state->getValue('search_page_enable')) {
      return;
    }
    if (!$form_state->getValue('search_page_path')) {
      $message = $this->t('%field: This field is required when the search page is enabled.', [
        '%field' => $this->t('Path'),
      ]);
      $form_state->setErrorByName('search_page_path', $message);
    }
    $js_config = $form_state->getValue('search_page_js_config');
    if (!$js_config) {
      $message = $this->t('%field: This field is required when the search page is enabled.', [
        '%field' => $this->t('Javascript config'),
      ]);
      $form_state->setErrorByName('search_page_js_config', $message);
    }
    elseif (strpos($js_config, 'studioConfig') === FALSE) {
      $message = $this->t('%field: The entered Javascript must at least define the "studioConfig" constant.', [
        '%field' => $this->t('Javascript config'),
      ]);
      $form_state->setErrorByName('search_page_js_config', $message);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('searchstax.settings')
      ->set('analytics_key', $form_state->getValue('analytics_key'))
      ->set('search_specific_analytics_keys', $form_state->getValue('search_specific_analytics_keys'))
      ->set('autosuggest_core', $form_state->getValue('autosuggest_core'))
      ->set('searches_via_searchstudio', $form_state->getValue('searches_via_searchstudio'))
      ->set('search_page_enable', $form_state->getValue('search_page_enable'))
      ->set('search_page_path', $form_state->getValue('search_page_path'))
      ->set('search_page_js_config', $form_state->getValue('search_page_js_config'))
      ->save();

    parent::submitForm($form, $form_state);

    // Rebuild the routing information, in case the search page configuration
    // changed.
    \Drupal::getContainer()->get('router.builder')->setRebuildNeeded();
  }

}
